/*
************************************************************************
*
*   POVView.c - POV viewing parameters
*
*   Copyright (c) 1994
*
*   ETH Zuerich
*   Institut fuer Molekularbiologie und Biophysik
*   ETH-Hoenggerberg
*   CH-8093 Zuerich
*
*   SPECTROSPIN AG
*   Industriestr. 26
*   CH-8117 Faellanden
*
*   All Rights Reserved
*
*   Date of last modification : 96/03/02
*   Pathname of SCCS file     : /sgiext/molmol/sg/src/pov/SCCS/s.POVView.c
*   SCCS identification       : 1.7
*
************************************************************************
*/

#include "pov_view.h"
#include "pov_clip.h"

#include <math.h>

#include <mat_vec.h>
#include <sg.h>
#include <sg_plot.h>
#include <sg_get.h>
#include "pov_util.h"

#define BIG_FACT 1000.0

static SgProjection Proj;
static float ViewPoint[3], ViewNorm[3], ViewDist;
static float ViewAng;
static float Near, Far;

void
SgPOVUpdateView(void)
{
  float x, y, w, h;

  SgGetViewport(&x, &y, &w, &h);

  SgPlotOutputStr("camera {\n");

  if (Proj == SG_PROJ_ORTHO) {
    SgPlotOutputStr("location <");
    SgPlotOutputFloat(BIG_FACT * ViewPoint[0]);
    SgPlotOutputStr(", ");
    SgPlotOutputFloat(BIG_FACT * ViewPoint[1]);
    SgPlotOutputStr(", ");
    SgPlotOutputFloat(BIG_FACT * ViewPoint[2]);
    SgPlotOutputStr(">\n");
    SgPlotOutputStr("direction <0, 0, ");
    SgPlotOutputFloat(BIG_FACT * 0.5 / tanf(0.5 * ViewAng));
    SgPlotOutputStr(">\n");
  } else {
    SgPlotOutputStr("location ");
    POVPlotVect(ViewPoint);
    SgPlotOutputStr("\n");
    SgPlotOutputStr("direction <0, 0, ");
    SgPlotOutputFloat(0.5 / tanf(0.5 * ViewAng));
    SgPlotOutputStr(">\n");
  }

  if (SgPlotGetOrient() == SG_PLOT_ROT90) {
    SgPlotOutputStr("up <0, ");
    SgPlotOutputFloat(w / h);
    SgPlotOutputStr(", 0>\n");
    SgPlotOutputStr("right <-1, 0, 0>\n");
    SgPlotOutputStr("sky <1, 0, 0>\n");
  } else {
    SgPlotOutputStr("right <");
    SgPlotOutputFloat(- w / h);
    SgPlotOutputStr(", 0, 0>\n");
  }

  SgPlotOutputStr("look_at <0, 0, 0>\n");
  SgPlotOutputStr("}\n");

  /* clip volume */
  SgPlotOutputStr("declare C = difference {\n");
  SgPlotOutputStr("plane {");
  POVPlotVect(ViewNorm);
  SgPlotOutputStr(", ");
  SgPlotOutputFloat(ViewDist - Near);
  SgPlotOutputStr("}\n");
  SgPlotOutputStr("plane {");
  POVPlotVect(ViewNorm);
  SgPlotOutputStr(", ");
  SgPlotOutputFloat(ViewDist - Far);
  SgPlotOutputStr("}\n");
  SgPlotOutputStr("}\n");
}

void
SgPOVSetProjection(SgProjection proj)
{
  Proj = proj;
}

void
SgPOVSetViewPoint(float x[3])
{
  Vec3Copy(ViewPoint, x);
  ViewDist = Vec3Abs(x);
  Vec3Copy(ViewNorm, x);
  Vec3Scale(ViewNorm, 1.0 / ViewDist);
}

void
SgPOVSetViewAngle(float ang)
{
  ViewAng = ang;
}

void
SgPOVSetNearPlane(float d)
{
  Near = d;
}

void
SgPOVSetFarPlane(float d)
{
  Far = d;
}

BOOL
POVClip(float x[3], float r)
{
  float d;

  d = Vec3Scalar(x, ViewNorm);
  if (d + r > ViewDist - Near || d - r < ViewDist - Far) {
    SgPlotOutputStr("clipped_by {C}\n");
    return FALSE;
  } else {
    return TRUE;
  }
}
